--  Telemtry script: RES Training App
--  #################################
--
--  version 9   30.07.2021
--  by ChrisOHara
--  OpenTX version 2.3.7
--  for Horus and Taranis radios (but may run on other openTX radios)
--  This Software is free
--
--  Information: Using a barometer this software is not allowed in official RES competion.
--
--  Requirements: Telemetry value "Alt" (so a barometer is required)
--                Momentary switch "sf" and "sg"
--                   (switches can be changed by altering software definitions, switchname_one and switchname_two)
-- 
--
--  Reaching 5m (starting height) you will hear a beep (1000 Hz).
--  During the following 10 seconds you can start the timer by operating momentary switch "sf" or "sg"
--     when plane is released from leash.
--  If you do not operate a switch  the  timer will start after 10 seconds on its own.
--  When timer starts you can hear a beep (2000Hz).
--
--  reaching landing height (2m) there is another beep (1000Hz).
--  During the next 10 seconds you can stop the timer by operating momentary switch "sf" or "sg"
--     when having contact with ground.
--  If you do not operate a switch the timer timer will stop after 10 seconds on its own.
--  When timer stops you can hear a beep (2000Hz).
--
--  Hint: If you do not use a swich  you will get the flighttime between 
--        5m (starting altidude) and 2m (landing altidude).
--
--  The script does use/define timer 1 on its own. (6 minutes, countdown)
--
--  There will be displayed the last flights. 
--      Horus: max is 2,6,8,19 depending on screen zone you select for widget
--      Taranis: max is 8
--
--  Information displayed:
--      Flight     =    number of flight (the newest flight is always number 0)
--      Time       =    Flighttime in seconds
--      Alt-S      =    starting altidude
--      Alt-F      =    maxmal altidude (during flight from start timer to stop timer)
--
--  Installation:
--      Horus: Copy /WIDGETS/res_H/main.lua and /SCRIPTS/TELEMETRY/res_H.lua   to the sd_card of your Horus.
--             Add the widget res_H to your telemetry screen.
--      Taranis: Copy /SCRIPTS/TELEMETRY/res_T.lua   to the sd_card of your Taranis.
--               Add the script res_T to your telemetry screen.
--
--  Errors: Scripts aborts when function getValue() is called with nil.
--  reason: Telemetry value "Alt" is not available or
--          one or both switches "sf" or "sg" are not available.(change script, define other switches
--          in "switchname_one" and/or "switchname_two")
--
-- 
--  Telemetry Script:  Trainingsprogramm für RES Flieger
--  ####################################################
--
--  version 9   30.07.2021
--  von ChrisOHara
--  OpenTX version 2.3.7
--  für Horus and Taranis radios (läuft aber eventuell auch auf anderen openTX Radios)
--  diese Software ist frei
--
--  Hinweis: Da ein Barometer verwendet wird, ist die Software nicht in einem RES Wettbewerb erlaubt.
--
--  Voraussetzungen:
--           Benötigt wird der Telemetriewert "alt"  (Höhe/Barometer).
--           Momentary Schalter "sf" oder "sg"
--             (kann im script verändert werden über switchname_one und switchname_two)
--  
--  Bei 5m (Starthöhe) ertönt ein Beep (1000 Hz).
--  Innerhalb der nächsten zehn Sekunden kann man durch Drücken der Tasten "sf" 
--    oder "sg" (beim Ausklinken des Seils) den Timer starten.
--  Drückt man keine Taste, wird der Timer automatisch nach zehn Sekunden gestartet.
--  Wenn der Timer startet, ertönt ein Beep (2000 Hz).
--  
--  Bei 2m (Landehöhe) ertönt wieder ein Beep (1000 Hz).
--  Innerhalb der nächsten zehn Sekunden kann man (bei Bodenberührung) durch Drücken der Tasten "sf" 
--    oder "sg" den Timer stoppen.
--  Drückt man keine Taste, wird der Timer automatisch nach zehn Sekunden gestoppt.
--  Wenn der Timer stopped, ertönt ein Beep (2000 Hz).
--  
--  Hinweis: Verzichtet man auf das Drücken der Tasten erhält man demzufolge die Zeit zwischen 
--           Starthöhe (5m) und Landehöhe (2m).
--  
--  Das Programm verwendet/definiert automatisch den Timer 1. (6 Minuten, countdown)
--
--  Es werden die Daten der letzen Flüge angezeigt. 
--      Horus: maximal 2,6,8,19 abhängig vom gewählen Bildschirmausschnitt für das Widget
--      Taranis: maximal 8
--
--  Installation:
--      Horus: Kopiere /WIDGETS/res_H/main.lua und /SCRIPTS/TELEMETRY/res_H.lua   auf die sd_card deiner Horus.
--             Füge das Widget res_H deinem Telemetry Screen hinzu.
--      Taranis: Kopiere /SCRIPTS/TELEMETRY/res_T.lua   auf die sd_card deiner Taranis.
--               Füge das Script res_H deinem Telemetry Screen hinzu.               
--
--  Angezeigte Infomation:
--	Flight    =    Flugnummer (der jüngste Flug ist immer 0)
--	Time      =    Flugzeit in Sekunden
--      Alt-S     =    Starthöhe (starten des Timers)
--      Alt-F     =    maximale Flughöhe (während des gesamten Fluges zwischen Timer-Start und Timer-Stop)
--
--  Fehler: Das Script bricht ab, wenn die Funktion getValue() mit "nil" aufgerufen wird.
--  Grund:  Der Telemetrie Wert "Alt" ist nicht verfügbar oder
--          einen oder beide Schalter "sf" oder "sg" gibt es nicht.(Script ändern, in "switchname_one"
--          und "switchname_two" andere Schalter eintragen) 
--
--  #######################################################################################################



local my_start_altitude = 5;       -- the altitude to recognize starting condition in [m]
local landing_altitude = 2;        -- the altitude to recognize landing condition in [m]
local start_stop_time_delta = 10;
local my_timer_number = 0;	   -- the timer that is resetted by start condition

local switch_condition_start = -1024;
local switch_condition_stop = -1024;
local switchId_one = nil;
local switchId_two = nil;
local switchname_one = "sf";
local switchname_two = "sg";
local my_timer = nil;
local start_time;
local flight_time_start_end_limit;
local altitude = 0;
local max_altitude = 0;
local altId = nil;
local my_condition = 0;          -- 0="looking for start", 1="looking for flying"; 2="looking for landing"; 3="looking for stop timer"

local max_flight_counter = 8;    -- maximum number of flights stored, prevents screen overflow
local flight_count = 0;          -- counts flights stored
local i=0;
local flight_times = {};
local start_altitudes = {};
local flight_altitudes = {};



-- ############################# init function ##################
local function init_func()
    altId = getFieldInfo("Alt") and getFieldInfo("Alt").id or nil;
    switchId_one = getFieldInfo(switchname_one) and getFieldInfo(switchname_one).id or nil;  --  defines switch  as switch for time meassurement
    switchId_two = getFieldInfo(switchname_two) and getFieldInfo(switchname_two).id or nil;  --  defines switch  as switch for time meassurement
    my_timer = model.getTimer(my_timer_number);
      my_timer.mode = 0;
      my_timer.start = 360;
      my_timer.value = 360;
      my_timer.countdownBeep = 2;
      my_timer.minuteBeep = false;
      my_timer.persistant = 0;
    model.setTimer(my_timer_number , my_timer);
end



-- #############################  looking for start #############
local function look_for_start()                                     -- called as long as my_condition is 0
   altitude=getValue(altId);
   if altitude > my_start_altitude then 
     max_altitude = altitude;
     for i = (max_flight_counter-1),1,-1 do    -- shift Globalvariables to empty first slot for newer values
        flight_times [i] = flight_times [i-1];
        start_altitudes [i] = start_altitudes [i-1];
        flight_altitudes [i] = flight_altitudes [i-1];
       end
     max_altitude = 0;
     start_altitudes[0]=0;
     flight_times[0] = 0;
     flight_altitudes[0] = 0;
     if flight_count < max_flight_counter then
        flight_count = flight_count + 1;
       end
     flight_time_start_end_limit = getTime() + start_stop_time_delta*100;
     switch_condition_start = -1024; 
     switch_condition_stop  = -1024;
     playTone(1000, 500, 500, PLAY_NOW); 
     my_condition = 1;
   end
end

-- #############################  looking for flying (release from leach) ############
local function look_for_flying()                                   -- called as long as my_condition is 1

   altitude=getValue(altId);
   if altitude > max_altitude then 
	max_altitude = altitude;
        start_altitudes[0] = (math.floor(max_altitude*10 + 0.5)) /10;
	flight_altitudes[0] = start_altitudes[0];
     end

   start_time=getTime();

   if ((getValue(switchId_one) == 1024) or (getValue(switchId_two) == 1024)) then
	switch_condition_start=1024;
     end
   if ((start_time >= flight_time_start_end_limit) or (switch_condition_start == 1024) ) then
        my_timer.mode = 1;
        model.setTimer(my_timer_number , my_timer);
	playTone(2000, 500, 500, PLAY_NOW);
	my_condition = 2;
     end

end

-- #############################  looking for landing ############
local function look_for_landing()                                   -- called as long as my_condition is 2
   altitude=getValue(altId);
   if altitude > max_altitude then 
	max_altitude = altitude;
        flight_altitudes[0] = (math.floor(max_altitude*10 + 0.5)) /10;
     end
    flight_times[0] = math.floor((getTime() - start_time) / 100 + 0.5);
   if altitude < landing_altitude then
        flight_time_start_end_limit = flight_times[0] + start_stop_time_delta;
	playTone(1000, 500, 500, PLAY_NOW);
	my_condition = 3;
     end
end

-- ############################# looking for stop timer ###########
local function look_for_stop_timer()                                 -- called as log as my_condition is 3
   flight_times[0] = math.floor((getTime() - start_time) / 100 + 0.5);
   if ( (getValue(switchId_one) == 1024) or (getValue(switchId_two) == 1024) ) then
	switch_condition_stop=1024; 
     end
   if ((flight_times[0] >= flight_time_start_end_limit) or (switch_condition_stop == 1024) ) then
        my_timer.mode = 0;
        model.setTimer(my_timer_number , my_timer);
	playTone(2000, 500, 500, PLAY_NOW);
	my_condition = 0;
     end
end

-- ############################### background function #################
local function bg_func()
  if       my_condition == 0 then look_for_start();
    elseif my_condition == 1 then look_for_flying();
    elseif my_condition == 2 then look_for_landing();
    elseif my_condition == 3 then look_for_stop_timer();
    end
end

-- ############################### run function #################
local function run_func()
    lcd.clear()
    lcd.drawText (5,0,"Flight Time    Alt-S    Alt-F", SMLSIZE+INVERS)

    for i=0,(flight_count-1) do
	local xx = 8+i*7;
	lcd.drawText (12,xx,i, SMLSIZE)
	lcd.drawText (40,xx,flight_times[i], SMLSIZE)
	lcd.drawText (70,xx,start_altitudes[i], SMLSIZE)
	lcd.drawText (105,xx,flight_altitudes[i], SMLSIZE)
        end
    if my_condition > 0 then lcd.drawText (5,8,"flying", SMLSIZE+BLINK); end

end


return { run=run_func, background=bg_func, init=init_func }
